#include <rpc/rpc.h>
#include <powerrpc.h>

#define NETBULA_RPC_DEFAULT_TIMEOUT 25

class RPCClient {

protected:

	CLIENT * clnt;
	struct timeval timeout;

public:

	RPCClient () {
			clnt = NULL;
			timeout.tv_sec=NETBULA_RPC_DEFAULT_TIMEOUT;
			timeout.tv_usec=0;
	}

	
   	 RPCClient(const char *hostname, const char *proto, unsigned long prog, unsigned long vers){
		
		clnt = clnt_create(hostname?hostname:"localhost", prog, vers, proto?proto:"tcp");
		timeout.tv_sec=NETBULA_RPC_DEFAULT_TIMEOUT;
		timeout.tv_usec=0;
		return;
	}

	inline void setTimeout(struct timeval to) {
		timeout = to;
	}

	inline CLIENT * getHandle() {
		return clnt;
	}

	

	inline CLIENT *setHandle(CLIENT *h) {
		CLIENT * oh = clnt;
		clnt = h;
		return oh;
	}

#if defined(WIN32)
	inline void abort(){
		if(clnt) clnt_abort(clnt);
	}
#endif


	inline void geterr(struct rpc_err *errp) {
              if(clnt) clnt_geterr(clnt,errp);
	}

        inline bool_t freeres(xdrproc_t xres,caddr_t resp) {
		if(clnt) return clnt_freeres(clnt, xres, resp);
		return 0;
	}

	inline bool_t control (u_int request, char*info) {
		if(clnt) return clnt_control(clnt, request, info);
		return 0;
	
	}

	inline int getSocket() {
		int sock=-1;
		if(clnt) clnt_control(clnt, CLGET_FD, (char*)&sock);
		return sock;
	}

	inline void destroy () {
		if(clnt) {
			clnt_destroy(clnt);
			clnt = NULL;
		}
	}


	inline void finish () {
		destroy();
	}


	inline CLIENT * raw_create(u_long prog, u_long vers) {
		clnt = clntraw_create(prog, vers);
		return clnt;
	}



    inline CLIENT * create(const char *hostname, unsigned prog, unsigned vers, char *proto){
		clnt = clnt_create(hostname, prog, vers, proto);
		return clnt;
	}

	//Specify port number, by pass pmap query
	inline CLIENT *create2(const char*hostname,  u_long prog, u_long vers, 
					char* proto, short port=0, u_int sendsz=0, u_int recvsz=0){

	    clnt = pw_clnt_create(hostname, port, prog, vers, proto,  sendsz,  recvsz);
		return clnt;
	}


	inline CLIENT *tcp_create( struct sockaddr_in *raddr,
	u_long prog,
	u_long vers,
	int *sockp,
	u_int sendsz,
	u_int recvsz
	) 
	{

	clnt = clnttcp_create(raddr,prog,vers,sockp, sendsz, recvsz);
	return clnt;
	}



	inline CLIENT * udp_create(
		struct sockaddr_in *raddr,
		u_long program,
		u_long version,
		struct timeval wait,
		int *sockp)
	{

		clnt = clntudp_create(raddr,program,version,wait,sockp);
		return clnt;
	}



	inline CLIENT *udp_bufcreate(
			struct sockaddr_in *raddr,
		u_long program,
		u_long version,
		struct timeval wait,
		int *sockp,
		u_int sendsz,
		u_int recvsz)
	{

		clnt = clntudp_bufcreate(raddr,program,version,wait,sockp,sendsz,recvsz);
		return clnt;
	}


	// call rpc
 
	enum clnt_stat 
	   call(u_long proc, xdrproc_t xargs, caddr_t argsp, 
			xdrproc_t xres, caddr_t resp, struct timeval timeout){
		if(clnt) {
			return clnt_call(clnt, proc, xargs, argsp, xres, resp, timeout);
		}
		return RPC_FAILED;
	}




/*
 * Print why creation failed
 */

	inline void pcreateerror(char* s){
		clnt_pcreateerror(s);
	}

	inline char * spcreateerror(char*s){
		return clnt_spcreateerror(s);
	}

/*
 * Like clnt_perror(), but is more verbose in its output
 */ 
	inline void perrno( enum clnt_stat num){
		clnt_perrno(num);
	}

/*
 * Print an English error message, given the client error code
 */

	inline void perror(char *s){
		if(clnt) clnt_perror(clnt,s);
	};


	inline char * sperror(char *s){
		if(clnt) return clnt_sperror(clnt ,s);
		return NULL;
	}




	char * sperrno( enum clnt_stat stat){
		return 
			clnt_sperrno(stat);
	}


#if defined(WIN32)
#ifndef _WIN32_WCE

	WSAEVENT  async_call(u_long proc, xdrproc_t xdr_args, caddr_t args_ptr){
			if(clnt) return tcp_async_call(clnt, proc, xdr_args, args_ptr);
			return 0;
	}

	enum clnt_stat async_call_getresult(
		xdrproc_t xdr_results,
		caddr_t results_ptr,
		struct timeval timeout){
		if(clnt) return  tcp_async_call_getresult(clnt,xdr_results,results_ptr,timeout);
		return RPC_FAILED;
	}

#endif
#endif


};
