#include "rfile.h"

#define exit_if_rpc_err() { if(RPC_SUCCESS!=rfile_errno(0)) exit(-1);}

int             get_file(char *src, char *local)
{
    rFILE          *fp = 0;
    FILE           *localfp;
    char            buf[1024];
    int             cnt;

    localfp = fopen(local, "wb");
    if (!localfp) {
        perror(local);
        return -1;
    }
    fp = rfopen(src, "rb");
    exit_if_rpc_err();

    if (fp == 0) {
        fprintf(stderr, "Fail to open remote file!\n");
        fclose(localfp);
        return -1;
    }
    while ((cnt = rfread(buf, 1, 1024, fp)) > 0) {
        exit_if_rpc_err();
        fwrite(buf, 1, cnt, localfp);
    }
    exit_if_rpc_err();

    fclose(localfp);
    rfclose(fp);
    exit_if_rpc_err();
    pw_free_reference((xdrproc_t)xdr_rFILE, (void*)fp, sizeof(*fp));
    return 0;
}

int             put_file(char *src, char *remote)
{
    rFILE          *fp;
    FILE           *localfp;
    char            buf[1024];
    int             cnt;

    localfp = fopen(src, "rb");
    if (!localfp) {
        perror(src);
        return -1;
    }
    fp = rfopen(remote, "wb");

    exit_if_rpc_err();
    if (fp == 0) {
        fprintf(stderr, "Fail to open remote file!\n");
        fclose(localfp);
        return -1;
    }
    while ((cnt = fread(buf, 1, 1024, localfp)) > 0) {
        rfwrite(buf, 1, cnt, fp);
        exit_if_rpc_err();
    }
    exit_if_rpc_err();

    fclose(localfp);
    rfclose(fp);
    exit_if_rpc_err();
    pw_free_reference((xdrproc_t)xdr_rFILE, (void*)fp, sizeof(*fp));
    return 0;
}

int             list_remote_dir(char *path)
{
    void            free_entry(DENTRY * pentry);
    void            print_entry(FILE * fp, DENTRY * pdentry);
    DENTRY          direntry;
    int             ret;
    direntry.next = 0;
    ret = rlistdir(path, &direntry);
    exit_if_rpc_err();
    print_entry(stdout, &direntry);
    DENTRY_freeres(&direntry);
    return ret;
}




enum {
    INVALID, GET, PUT, QUIT, LIST, CD
};

struct {
    int             cmd_type;
    char           *cmd_str;
    char           *comment;
}               commands[] = {
    {
        GET, "get", "get a remote file"
    },
    {
        PUT, "put", "send local file to remote system"
    },
    {
        LIST, "ls", "list the contents of a directory"
    },
    {
        CD, "cd", "change remote working directory"
    },
    {
        QUIT, "quit", "exit"
    },
    {
        INVALID, 0, 0
    }
};

int             get_cmd_type(char *cmd_str)
{
    int             i;
    for (i = 0; commands[i].cmd_str != 0; i++) {
        if (!strcmp(commands[i].cmd_str, cmd_str))
            return commands[i].cmd_type;
    }
    return INVALID;
}

void            print_help()
{
    int             i;
    printf("Available commands\n");
    for (i = 0; commands[i].cmd_str != 0; i++) {
        printf("%s ==> %s\n", commands[i].cmd_str, commands[i].comment);
    }
}

int             do_cmd(char *buf)
{

    char            path1[1024];
    char            cmd[256];
    path1[0] = '\0';

    cmd[0] = '\0';
    if (sscanf(buf, "%s%s", cmd, path1) < 1)
        return 0;

    switch (get_cmd_type(cmd)) {

    case GET:
        if (strlen(path1) == 0) {
            printf("No file name supplied for GET.\n");
            return 0;
        }
        return get_file(path1, path1);

    case PUT:
        if (strlen(path1) == 0) {
            printf("No file name supplied for PUT.\n");
            return 0;
        }
        return put_file(path1, path1);

    case QUIT:
        rfile_unbind(0);
        exit(0);

    case CD:
        return rchdir(path1);

    case LIST:
        if (path1[0] == '\0')
            strcpy(path1, ".");
        return list_remote_dir(path1);

    default:
        print_help();
    }
    return 0;

}

main(int argc, char **argv)
{
    char            buf[1024];

    if (argc != 2) {
        fprintf(stderr, "Usage: %s hostname\n", argv[0]);
        exit(1);
    }
    if (rfile_bind(argv[1], 0, 0, 0) == 0) {
        fprintf(stderr, "Fail to connect to server!\n");
        exit(1);
    }
    fprintf(stderr, "connected to server!\n");
    while (1) {
        printf("%s> ", argv[0]);
        gets(buf);
        if (do_cmd(buf) == 0) {
                printf("command successful.\n");
        } else {
                printf("command failed.\n");
        }
    }
}
