/*************************************************************************
 This is the server library functions for our implementation of
 FT (file transfer).
*************************************************************************/

#include "rfile.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifndef WIN32
#include <dirent.h>
#include <unistd.h>
#else 
#include <direct.h>
#include <io.h>
#endif

#include <malloc.h>


#define MAXFILE 1024

/*************************************************************************
 We use a global array to keep track of all the files that we have
 opened on behalf of the client. The client will use the index to
 the approriate entry in read/write calls later.
*************************************************************************/

FILE           *fd_table[MAXFILE];

#define ENTER(x) { printf("%d Enter %s\n",getpid(), #x); }


void            init_fdtable(int c, char**arr)
{
    int             i;
    for (i = 0; i < MAXFILE; i++) {
        fd_table[i] = 0;
    }

}

/*************************************************************************
 Find a unoccupied entry in the global table.
*************************************************************************/

int             get_free_entry(void)
{
    int             i;
    for (i = 0; i < MAXFILE; i++)
        if (fd_table[i] == NULL)
            return i;
    return -1;                  /* no more entries */
}


/*************************************************************************
 Open a file and save the FILE* pinter in the global table, and return
 the index to the client.
*************************************************************************/

rFILE          *
                rfopen(str1024 path, str1024 mode)
{
    FILE           *fp;
    int             index;
    static rFILE    rf;

	ENTER(rfopen);
    fp = fopen(path, mode);

    if (fp == 0) {
        fprintf(stderr, "Fail open file %s\n", path);
        return NULL;
    }
    index = get_free_entry();
    if (index < 0)
        return NULL;

    fd_table[index] = fp;
    rf.fd = index;
    return &rf;
}


int             rfread(void *ptr, size_T size, size_T nm, rFILE * stream)
{
    FILE           *fp;
    int             index = stream->fd;
	int             sz;

    /* first check if the index is valid, if true, get the FILE pointer */
    if (index < 0 || index >= MAXFILE || (fp = fd_table[index]) == 0) {
        fprintf(stderr, "Invalid rFILE pointer!\n");
        return -1;
    }
    sz = (int)fread(ptr, size, nm, fp);
	/*printf("size got: %d\n", sz);*/
	return sz;
}

int             rfwrite(const void *ptr, size_T size, size_T nm, rFILE * stream)
{
    FILE           *fp;
    int             index = stream->fd;
    if (index < 0 || index >= MAXFILE || (fp = fd_table[index]) == 0) {
        fprintf(stderr, "Invalid rFILE pointer!\n");
        return -1;
    }
    return fwrite(ptr, size, nm, fp);
}

int             rfclose(rFILE * stream)
{
    FILE           *fp;
    int             index = stream->fd;
    if (index < 0 || index >= MAXFILE || (fp = fd_table[index]) == 0) {
        fprintf(stderr, "Invalid rFILE pointer!\n");
        return -1;
    }
	fd_table[index]=0;
    return fclose(fp);
}


void            free_entry(DENTRY * pentry);

void            print_entry(FILE * fp, DENTRY * pdentry);

static void     fill_entry(char *path, DENTRY * pd)
{
    struct stat     statbuf;

    strcpy(pd->name, path);
#ifndef WIN32
    if (lstat(path, &statbuf) < 0) {
        return;
    }
    switch (statbuf.st_mode & S_IFMT) {
    case S_IFIFO:
        strcat(pd->name, "|");
        break;

    case S_IFLNK:
        strcat(pd->name, "@");
        break;

    case S_IFSOCK:
        strcat(pd->name, "-->");
        break;

    case S_IFDIR:
        strcat(pd->name, "/");
        break;

    default:
        ;
    }
#endif

}

int             rlistdir(char* dirpath, DENTRY * pentry)
{
#ifndef WIN32
    struct stat     statbuf;
    DIR            *pdir;
    struct dirent  *pdent;
#else
	struct _finddata_t fileinfo;
	long hfile;
#endif

    DENTRY         *pcur_entry;
    static DENTRY  *old_entry = 0;
    int             ret;


    if (pentry == NULL)
        return -1;

    pcur_entry = pentry;

    free_entry(old_entry);
    old_entry = 0;

#ifndef WIN32
    ret = lstat(dirpath, &statbuf);
    if (ret < 0)
        return ret;

    if (S_ISDIR(statbuf.st_mode) == 0)
        return -1;

    if ((pdir = opendir(dirpath)) == NULL)
        return -2;

    while ((pdent = readdir(pdir)) != NULL) {
#else
	if((hfile=_findfirst("*.*", &fileinfo))== -1L){
			printf("No files found\n");
			return -1;
	}

       
	do  {

#endif

        if (old_entry == 0) {
            old_entry = pcur_entry;
        } else {
            pcur_entry->next = (DENTRY *) malloc(sizeof(DENTRY));
            pcur_entry = pcur_entry->next;
            pcur_entry->next = NULL;
        }
#ifndef WIN32
        fill_entry(pdent->d_name, pcur_entry);
	 };
#else
		fill_entry(fileinfo.name, pcur_entry);
	 }while(_findnext(hfile, &fileinfo)==0);
#endif
     
    old_entry = old_entry->next;
#ifdef WIN32
	return _findclose(hfile);
#else
    return closedir(pdir) ;
#endif
}

int             rchdir(char *path)
{
    return chdir(path);
}
